"
 Copyright (c) 2007-2010 Jan Vrany
 Copyright (c) 2009-2010 eXept Software AG

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the 'Software'), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:libsvn' }"

"{ NameSpace: SVN }"

FileoutLikeTask subclass:#CommitTask
	instanceVariableNames:'message paths'
	classVariableNames:''
	poolDictionaries:''
	category:'SVN-Tasks'
!

!CommitTask class methodsFor:'documentation'!

copyright
"
 Copyright (c) 2007-2010 Jan Vrany
 Copyright (c) 2009-2010 eXept Software AG

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the 'Software'), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.

"
! !

!CommitTask class methodsFor:'others'!

version_CVS
    ^ '$Header$'
! !

!CommitTask methodsFor:'accessing'!

message

    message ifNil:[
        | changedClasses changedMethods |
        changedClasses := OrderedSet new.
        changedMethods := OrderedSet new.
        (ChangeSet current changesForPackage: package) do:
            [:chg| | chgCls |        
            chg isClassChange ifTrue:
                [(chgCls := chg changeClass theNonMetaclass) package == package
                    ifTrue:
                        [(classes isNil or:[classes includes: chgCls]) ifTrue:
                            [changedClasses add: chgCls]]
                    ifFalse:
                        [chg isMethodDefinitionChange ifTrue:
                            [changedMethods add: chg changeMethod]]]].
        message := String streamContents:
            [:s|
            s cr.
            changedClasses isEmpty ifFalse:
                [s nextPutLine:'## Changed classes:'.
                changedClasses do:
                    [:cls|s nextPutAll:'##   ';nextPutAll: cls name; cr]].
            changedMethods isEmpty ifFalse:
                [s nextPutLine:'## Changed methods:'.
                changedMethods do:
                    [:m|
                    s   nextPutAll:'##   '; nextPutAll: m mclass name; 
                        nextPutAll:' >> #'; nextPutAll: m selector;  cr]].     
            s nextPutLine:'##'.
            s nextPutLine:'## Lines starting with ## will be removed'.
    ]].
    ^message.

    "Modified: / 01-08-2010 / 13:02:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

message:aString
    message := aString.
!

paths
    ^ paths
!

paths:aCollection
    paths := aCollection.
! !

!CommitTask methodsFor:'executing'!

do
    self
        doPrepareWorkingCopy;
        doCommit

    "Created: / 23-03-2009 / 11:15:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 17-06-2009 / 10:16:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doCommit
    |containers commitInfo msg|

    msg := ((message ? '<no commit message>') asStringCollection
                reject: [:line|line size >= 2 and:[line first == $# and:[line second == $#]]])
                asString.

    self do:[
        ActivityNotification notify:'Commiting ' , (self package ? '').
        containers := paths notNil ifTrue:[
            paths
        ] ifFalse:[
            self isSelectiveFileoutTask ifTrue:[
                self containersToFileOut
            ] ifFalse:[ 
                #() 
            ]
        ].
        self 
            synchronized:[
                commitInfo := (CommitCommand new)
                            workingCopy:self workingCopy;
                            message: msg;
                            paths:containers;
                            execute.
                 "Update the working copy. We need svn info
                 to report commited revision"
                (UpdateCommand new)
                    workingCopy:self workingCopy;
                    execute
            ].
        self doCompileSvnRevisionNrMethod:true.
        self doCompileSvnRepositoryUrlStringMethod.        
        ActivityNotification notify:'Shrinking changes'.
        (ChangeSet current)
            condenseChangesForPackage2:self package;
            condenseChangesForExtensionsInPackage:self package;
            flushChangedClassesCache;
            yourself.
    ].
    self workingCopy commited.
    ^ commitInfo

    "Created: / 11-04-2008 / 09:20:01 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 19-08-2009 / 12:27:44 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 08-02-2012 / 18:52:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doPrepareWorkingCopy

    self package notNil ifTrue:[
        self do:[
            self workingCopy ensureIsValid.
            self doFileOutAll
        ]
    ].

    "Created: / 11-04-2008 / 09:19:27 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 17-08-2009 / 18:28:34 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 09-02-2012 / 10:30:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CommitTask class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_SVN
    ^ 'Id: SVN__CommitTask.st 371 2011-09-28 18:47:07Z vranyj1 '
! !
